import sqlite3

class KeyValueStore(dict):
    def __init__(self, filename=None):
        self.conn = sqlite3.connect(filename)
        self.conn.execute("CREATE TABLE IF NOT EXISTS kv (key text unique, value text)")

    def close(self):
        self.conn.commit()
        self.conn.close()

    def __len__(self):
        rows = self.conn.execute('SELECT COUNT(*) FROM kv').fetchone()[0]
        return rows if rows is not None else 0

    def iterkeys(self):
        c = self.conn.cursor()
        for row in self.conn.execute('SELECT key FROM kv'):
            yield row[0]

    def itervalues(self):
        c = self.conn.cursor()
        for row in c.execute('SELECT value FROM kv'):
            yield row[0]

    def iteritems(self):
        c = self.conn.cursor()
        for row in c.execute('SELECT key, value FROM kv'):
            yield row[0], row[1]

    def keys(self):
        return list(self.iterkeys())

    def values(self):
        return list(self.itervalues())

    def items(self):
        return list(self.iteritems())

    def __contains__(self, key):
        return self.conn.execute('SELECT 1 FROM kv WHERE key = ?', (key,)).fetchone() is not None

    def __getitem__(self, key):
        item = self.conn.execute('SELECT value FROM kv WHERE key = ?', (key,)).fetchone()
        if item is None:
            raise KeyError(key)
        return item[0]

    def __setitem__(self, key, value):
        self.conn.execute('REPLACE INTO kv (key, value) VALUES (?,?)', (key, value))

    def __delitem__(self, key):
        if key not in self:
            raise KeyError(key)
        self.conn.execute('DELETE FROM kv WHERE key = ?', (key,))

    def __iter__(self):
        return self.iterkeys()



import sys
import time
import logging
import os
import sys

from watchdog.observers import Observer
from watchdog.events import LoggingEventHandler,FileSystemEventHandler,FileSystemEvent,FileModifiedEvent

class Watcher(FileSystemEventHandler):
    def __init__(self,root_path:str):
        super(Watcher, self).__init__()
        self.root=root_path
        self.main_file=os.path.join(root_path,'main.py')
    def run(self,file):
        os.chdir(self.root)
        cmd='%s %s' % (sys.executable, file)
        os.popen(cmd)
    def start(self):
        self.run(self.main_file)
        self.watch()
    def on_modified(self, event:FileModifiedEvent):
        print('Modified:',event.src_path)
        if event.src_path.endswith('.py') and os.path.samefile(event.src_path,self.main_file):
            self.run(event.src_path)
    def watch(self):
        observer = Observer()
        observer.schedule(self, self.root, recursive=True)
        observer.start()
        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            observer.stop()
        observer.join()


if __name__ == "__main__":
    watcher=Watcher(r'D:\Projects\PythonProjects\pywinschedule\demo\run')
    watcher.start()


from .sqlitekvstore import KeyValueStore

class DBList:
    def __init__(self,path):
        self.kv=KeyValueStore(path)
    def commit(self):
        self.kv.commit()
    def close(self):
        self.kv.close()
    def clear(self):
        for key in self.kv.keys():
            self.remove(key)
    def list(self):
        return list(self.kv.keys())
    def add(self,record):
        self.kv[record]='1'
    def remove(self,record):
        del self.kv[record]
    def contains(self,record):
        return record in self.kv.keys()


from dbutils import DBList
import datetime
import os
from pywinschedule.config import HOME
DB_PATH=os.path.join(HOME,'run_once_a_day.db')

dblist=DBList(DB_PATH)
def run_if_have_not_been_run_today(func,tag):
    today=datetime.datetime.today().date().strftime('%Y-%m-%d')+'-'+tag
    if dblist.contains(today):
        return
    else:
        func()
        dblist.add(today)
        dblist.commit()


